#!/usr/bin/env /usr/bin/python
import sys
import numpy as np
import matplotlib, pandas
matplotlib.use('Agg')
import matplotlib.pyplot as plt

if (len(sys.argv) < 3):

    print('\n This script converts the distance distribtuion in CASP format into distance map based on weighted average, \n and visualizes the distance map with the matplotlib module.\n')
    print(' Usage:  python visualize_pre.py predict_file native_file\n')
    print(' Example: python visualize_pre.py seq.dist seq.pdb \n The output file is seq.predist.png\n')
    exit(1)


prefile=sys.argv[1]
pdbfile=sys.argv[2]

matplotlib.rcParams['image.cmap'] = 'gist_stern'

def weighted_avg_and_std(values, weights):
    if(len(values)==1):return values[0],0

    average = np.average(values, weights=weights)
    variance=np.array(values).std()
    return average, variance

def re_pre(prefile,pdbfile):
	#read and renumber the predicted distance matrix
	ress=[]
	with open(pdbfile,'r') as f:
		for line in f:
			line=line.strip()
			if line[0:4] == "ATOM":
				resnum=line[22:26].strip()
				if int(resnum) not in ress:
					ress.append(int(resnum))
		
	map={}

	for i in range(len(ress)):
		map[ress[i]]=i

	prematrix=np.zeros((len(ress),len(ress),10),dtype=float)
			


	#predis=np.zeros((len(ress),len(ress),9),dtype=float)
	with open(prefile,'r') as f:
		for line in f:
			line=line.strip().split()
			if line==[]:
				continue
			if not line[0].isdigit():
				continue
			else:
				try:
					i=int(line[0])
					j=int(line[1])
					if i in map.keys() and j in map.keys():
							for k in range(10):
								prematrix[map[i]][map[j]][k]=prematrix[map[j]][map[i]][k]=float(line[3+k])

				except Exception as e:
					print(e)
					print("[error] your input file is not in standard CASP format!")
					sys.exit(1)
	return prematrix


def dist(dat):
    nres=int(dat.shape[0])

    pcut=0.05
    bin_p=0.01

    mat=np.zeros((nres, nres))

    
    for i in range(0, nres):
        for j in range(0, nres):
            
            if(j == i):
                mat[i][i]=4
                continue

            if(j<i):
                mat[i][j]=mat[j][i]
                continue

                       
            #check probability
            Praw = dat[i][j]
            first_bin=1
            first_d=5
            weight=0


            
            for P in Praw[first_bin:-1]:
                if(P>bin_p): weight += P
                
                
            if(weight < pcut):
                mat[i][j]=20
                continue


            Pnorm = [P for P in Praw[first_bin:-1]]

            probs=[]
            dists=[]
            xs = []
            ys = []
            dmax=0
            pmax=-1

            for k,P in enumerate(Pnorm):
                d = first_d + k*2
                if(P>pmax):
                    dmax=d
                    pmax=P
                #endif

                if(P>bin_p):
                    probs.append(P)
                    dists.append(d)
                #endif
                xs.append(d)

            e_dis=8;
            e_std=0;
            if(len(probs)==0):
                e_dis=dmax
                e_std=0;
            else:
                probs = [P/sum(probs) for P in probs]

                e_dis, e_std=weighted_avg_and_std(dists, probs)

            mat[i][j]=e_dis


    return mat



print("convert distance...")

#Distance
dat=re_pre(prefile,pdbfile)
plt.figure()
img=dist(dat)
color_map = plt.imshow(img.astype(float))
color_map.set_cmap("hot")
plt.colorbar()
name = 'seq.predist.png'    
plt.savefig(name, bbox_inches='tight', dpi=600, transparent=False)


print("The conversion is done.\n")

